#!/usr/bin/python3

"""Program to monitor Dovecot Director user threshold."""

import os
import sys
from typing import Dict, Optional, Tuple, Union

from cyberfusion.Common.Command import CyberfusionCommand

PERCENTAGE_CRITICAL_THRESHOLD = 80

LINE_INDEX_MAIL_SERVER_IP = 0
LINE_INDEX_VHOSTS = 2
LINE_INDEX_USERS = 5

DOVEADM_BIN = os.path.join(os.path.sep, *["usr", "bin", "doveadm"])


def get_percentage(part: Union[int, float], whole: Union[int, float]) -> float:
    """Get percentage."""
    return 100 * part / whole


def get_status_lines() -> str:
    """Get mail servers status lines."""
    return CyberfusionCommand(f"{DOVEADM_BIN} director status").stdout


def get_value_by_status_line(status_line: str) -> Tuple[str, int, int]:
    """Parse status line (mail server IP, vhosts, users)."""
    split = status_line.split()

    return (
        split[LINE_INDEX_MAIL_SERVER_IP],
        int(split[LINE_INDEX_VHOSTS]),
        int(split[LINE_INDEX_USERS]),
    )


def convert_status_lines_to_dict(
    status_lines: str,
) -> Dict[str, Dict[str, Union[int, float]]]:
    """Convert status lines string to dict.

    Each mail server IP is added to dict as key. Amount of vhosts, amount of
    users and corresponding percentage is added.
    """
    result: Dict[str, Dict[str, Union[int, float]]] = {}

    # Loop through status lines

    for status_line in status_lines.splitlines():
        # Skip first non-status line

        if status_line.split() == [
            "mail",
            "server",
            "ip",
            "tag",
            "vhosts",
            "state",
            "state",
            "changed",
            "users",
        ]:
            continue

        # Set values by parsing status line

        mail_server_ip, vhosts, users = get_value_by_status_line(status_line)

        # Add new mail server IP to dict

        if mail_server_ip not in result:
            result[mail_server_ip] = {}

        # Add values to dict

        result[mail_server_ip]["vhosts"] = vhosts
        result[mail_server_ip]["users"] = users
        result[mail_server_ip]["percentage"] = get_percentage(
            result[mail_server_ip]["users"],
            result[mail_server_ip]["vhosts"],
        )

    return result


def check_user_threshold(
    status_lines_dict: Dict[str, Dict[str, Union[int, float]]]
) -> Tuple[bool, Optional[str]]:
    """Determine if any mail server exceeds user threshold.

    Stop loop when any mail server exceeds user threshold.
    """
    for mail_server_ip, values in status_lines_dict.items():
        if values["percentage"] < PERCENTAGE_CRITICAL_THRESHOLD:
            continue

        return (
            True,
            f"CRITICAL: {mail_server_ip} exceeds threshold of {values['percentage']}% ({values['users']} users of {values['vhosts']} vhosts)",  # noqa: E501
        )

    return (
        False,
        f"OK: All mail servers user threshold ({PERCENTAGE_CRITICAL_THRESHOLD}%)",  # noqa: E501
    )


if __name__ == "__main__":
    result = check_user_threshold(
        convert_status_lines_to_dict(get_status_lines())
    )

    exceeds_threshold, message = check_user_threshold(
        convert_status_lines_to_dict(get_status_lines())
    )

    print(message)  # noqa: T001

    if exceeds_threshold:
        sys.exit(2)

    sys.exit(0)
